// Copyright 2008-2018 Yolo Technologies, Inc. All Rights Reserved. https://www.comblockengine.com


#ifndef KBENGINE_DEF_PROPERTY_H
#define KBENGINE_DEF_PROPERTY_H

#include "common/common.h"
#if KBE_PLATFORM == PLATFORM_WIN32
#pragma warning (disable : 4910)
#pragma warning (disable : 4251)
#endif

#include "fixedarray.h"
#include "fixeddict.h"
#include "datatype.h"
#include "common/refcountable.h"
#include "common/memorystream.h"


namespace KBEngine{

class RefCountable;
class PropertyDescription : public RefCountable
{
public:	
	PropertyDescription(ENTITY_PROPERTY_UID utype, 
		std::string dataTypeName, 
		std::string name, 
		uint32 flags, 
		bool isPersistent, 
		DataType* dataType, 
		bool isIdentifier, 
		std::string indexType,
		uint32 databaseLength, 
		std::string defaultStr, 
		DETAIL_TYPE detailLevel);

	virtual ~PropertyDescription();
	
	/** 
		ȡԵdetailLevel 
	*/
	INLINE int8 getDetailLevel(void) const;
	
	/** 
		Ƿһ浽ݿе 
	*/
	INLINE bool isPersistent(void) const;
	INLINE void isPersistent(bool v);

	/** 
		ȡԵ 
	*/
	INLINE DataType* getDataType(void) const;
	
	/** 
		ȡԵı־ cell_public 
	*/
	INLINE uint32 getFlags(void) const;
	INLINE void setFlags(uint32 flags);

	/** 
		ȡ 
	*/
	INLINE const char* getName(void) const;
	
	/** 
		ȡַ UINT32, BAG..
	*/
	INLINE const char* getDataTypeName(void) const;
	
	/** 
		ȡʼֵַ
	*/
	INLINE const char* getDefaultValStr(void) const;

	/** 
		Ե ϴʶ 
	*/
	INLINE ENTITY_PROPERTY_UID getUType(void) const;
	
	/** 
		ȡ
	*/
	INLINE const char* indexType(void) const;

	/** 
		id ¶ķ߹㲥ܸС255ʱ
		ǲʹutypeʹ1ֽڵaliasID
	*/
	INLINE int16 aliasID() const;
	INLINE uint8 aliasIDAsUint8() const;
	INLINE void aliasID(int16 v);

	/** 
		Ϊ 
	*/
	INLINE void setIdentifier(bool isIdentifier);
	
	/** 
		ݿеĳ 
	*/
	INLINE void setDatabaseLength(uint32 databaseLength);
	INLINE uint32 getDatabaseLength() const;

	/** 
		ȡdefļбĬֵ 
	*/
	virtual PyObject* newDefaultVal(void);
	
	/** 
		 
	*/
	static uint32 getDescriptionCount(void){ return propertyDescriptionCount_; }
	static void resetDescriptionCount(void){ propertyDescriptionCount_ = 0; }

	/** 
		Ͳһʵ 
	*/
	static PropertyDescription* createDescription(ENTITY_PROPERTY_UID utype, 
		const std::string& dataTypeName,
		const std::string& name,
		uint32 flags, 
		bool isPersistent, 
		DataType* dataType, 
		bool isIdentifier, 
		std::string indexType,
		uint32 databaseLength,
		const std::string& defaultStr,
		DETAIL_TYPE detailLevel);
	
	/** 
		űԵֵ 
	*/
	virtual PyObject* onSetValue(PyObject* parentObj, PyObject* value);	

	virtual void addToStream(MemoryStream* mstream, PyObject* pyValue);
	virtual PyObject* createFromStream(MemoryStream* mstream);
	virtual PyObject* parseDefaultStr(const std::string& defaultVal);

	virtual bool isSameType(PyObject* pyValue);
	virtual bool isSamePersistentType(PyObject* pyValue);

	virtual void addPersistentToStream(MemoryStream* mstream, PyObject* pyValue);
	virtual PyObject* createFromPersistentStream(MemoryStream* mstream);

	INLINE bool hasCell(void) const;
	INLINE bool hasBase(void) const;
	INLINE bool hasClient(void) const;
	
protected:	
	static uint32				propertyDescriptionCount_;						// е	
	std::string					name_;											// Ե
	std::string					dataTypeName_;									// Եַ
	uint32						flags_;											// ԵһЩ־   cell_public
	bool						isPersistent_;									// Ƿһ洢ݿ
	DataType*					dataType_;										// Ե
	bool						isIdentifier_;									// Ƿһ
	uint32						databaseLength_;								// ݿеĳ
	ENTITY_PROPERTY_UID			utype_;											// Ե ϴʶ
	std::string					defaultValStr_;									// ԵĬֵ
	DETAIL_TYPE					detailLevel_;									// Եlod鼶 commonе:Եlod㲥ΧĶ
	int16						aliasID_;										// id ¶ķ߹㲥ܸС255ʱ ǲʹutypeʹ1ֽڵaliasID
	std::string					indexType_;										// ԵUNIQUE, INDEXֱӦáΨһͨ
};

class FixedDictDescription : public PropertyDescription
{
public:	
	FixedDictDescription(ENTITY_PROPERTY_UID utype, 
		std::string dataTypeName,
		std::string name, 
		uint32 flags, 
		bool isPersistent, 
		DataType* dataType, 
		bool isIdentifier, 
		std::string indexType,
		uint32 databaseLength, 
		std::string defaultStr, 
		DETAIL_TYPE detailLevel);

	virtual ~FixedDictDescription();
	
	/** 
		űԵֵ 
	*/
	PyObject* onSetValue(PyObject* parentObj, PyObject* value);	

	virtual void addPersistentToStream(MemoryStream* mstream, PyObject* pyValue);
	virtual PyObject* createFromPersistentStream(MemoryStream* mstream);

	typedef std::vector<std::pair<std::string, KBEShared_ptr<PropertyDescription> > > CHILD_PROPERTYS;
	
protected:
	CHILD_PROPERTYS childPropertys_;
};

class ArrayDescription : public PropertyDescription
{
public:	
	ArrayDescription(ENTITY_PROPERTY_UID utype, 
		std::string dataTypeName, 
		std::string name, 
		uint32 flags, 
		bool isPersistent, 
		DataType* dataType, 
		bool isIdentifier,
		std::string indexType,
		uint32 databaseLength, 
		std::string defaultStr, 
		DETAIL_TYPE detailLevel);

	virtual ~ArrayDescription();
	
	/** 
		űԵֵ 
	*/
	PyObject* onSetValue(PyObject* parentObj, PyObject* value);

	virtual void addPersistentToStream(MemoryStream* mstream, PyObject* pyValue);
	virtual PyObject* createFromPersistentStream(MemoryStream* mstream);
	
protected:	
};

class VectorDescription : public PropertyDescription
{
public:	
	VectorDescription(ENTITY_PROPERTY_UID utype, 
		std::string dataTypeName, 
		std::string name, 
		uint32 flags, 
		bool isPersistent, 
		DataType* dataType, 
		bool isIdentifier, 
		std::string indexType,
		uint32 databaseLength, 
		std::string defaultStr, 
		DETAIL_TYPE detailLevel, 
		uint8 elemCount);

	virtual ~VectorDescription();
	
	/** 
		űԵֵ 
	*/
	PyObject* onSetValue(PyObject* parentObj, PyObject* value);
	
protected:	
	uint8 elemCount_;
};

class EntityComponentDescription : public PropertyDescription
{
public:
	EntityComponentDescription(ENTITY_PROPERTY_UID utype,
		std::string dataTypeName,
		std::string name,
		uint32 flags,
		bool isPersistent,
		DataType* dataType,
		bool isIdentifier,
		std::string indexType,
		uint32 databaseLength,
		std::string defaultStr,
		DETAIL_TYPE detailLevel);

	virtual ~EntityComponentDescription();

	/**
		űԵֵ
	*/
	PyObject* onSetValue(PyObject* parentObj, PyObject* value);

	virtual bool isSamePersistentType(PyObject* pyValue);
	virtual void addPersistentToStream(MemoryStream* mstream, PyObject* pyValue);
	void addPersistentToStreamTemplates(ScriptDefModule* pScriptModule, MemoryStream* mstream);
	virtual PyObject* createFromPersistentStream(MemoryStream* mstream);
	PyObject* createFromPersistentStream(ScriptDefModule* pScriptModule, MemoryStream* mstream);

	virtual PyObject* createFromStream(MemoryStream* mstream);

	/**
		ȡdefļбĬֵ
	*/
	virtual PyObject* newDefaultVal(void);

	virtual PyObject* parseDefaultStr(const std::string& defaultVal);

protected:
};

}

#ifdef CODE_INLINE
#include "property.inl"
#endif
#endif // KBENGINE_DEF_PROPERTY_H

